ignored_word_in_tags = {
    "about",
    "alors",
    "anc",
    "apprenants",
    "auprès",
    "aussi",
    "avec",
    "available",
    "avoir",
    "beaucoup",
    "beau",
    "beaux",
    "belle",
    "belles",
    "bon",
    "bonne",
    "bonnes",
    "bons",
    "cette",
    "chaque",
    "chose",
    "command",
    "comme",
    "créé",
    "créée",
    "dans",
    "default",
    "des",
    "different",
    "directory",
    "elle",
    "elles",
    "est",
    "était",
    "example",
    "exemple",
    "faire",
    "faite",
    "faites",
    "fait",
    "faits",
    "first",
    "following",
    "for",
    "from",
    "how",
    "however",
    "http",
    "https",
    "ils",
    "import",
    "included",
    "information",
    "jusqu",
    "learn",
    "les",
    "leur",
    "leurs",
    "moins",
    "name",
    "named",
    "never",
    "note",
    "notre",
    "other",
    "parce",
    "petit",
    "petits",
    "petite",
    "petites",
    "plus",
    "pour",
    "project",
    "quand",
    "quel",
    "quels",
    "quelle",
    "quelles",
    "quelque",
    "quelques",
    "readme",
    "réussie",
    "should",
    "section",
    "source",
    "specify",
    "super",
    "sur",
    "system",
    "that",
    "the",
    "there",
    "these",
    "this",
    "tous",
    "tout",
    "un",
    "une",
    "use",
    "using",
    "version",
    "votre",
    "voulez",
    "vous",
    "which",
    "with",
    "you",
    "your",
}
