simple_html_tags = {
    "a",
    "b",
    "bdi",
    "blockquote",
    "br",
    "code",
    "del",
    "details",
    "div",
    "em",
    "figcaption",
    "h1",
    "h2",
    "h3",
    "h4",
    "h5",
    "h6",
    "h7",
    "i",
    "ins",
    "li",
    "ol",
    "p",
    "picture",
    "pre",
    "small",
    "source",
    "span",
    "strong",
    "sub",
    "summary",
    "sup",
    "table",
    "td",
    "textarea",
    "th",
    "tr",
    "u",
    "ul",
}
