svg_tags = {
    "area",
    "circle",
    "defs",
    "fecolormatrix",
    "fecomponenttransfer",
    "fecomposite",
    "fefunca",
    "fefuncb",
    "fefuncg",
    "fefuncr",
    "filter",
    "g",
    "map",
    "path",
    "polygon",
    "polyline",
    "svg",
    "symbol",
}
